package com.diagbot.web;

import com.diagbot.dto.RespDTO;
import com.diagbot.entity.node.BillConcept;
import com.diagbot.entity.node.BillCondition;
import com.diagbot.entity.node.BillItem;
import com.diagbot.facade.BillManFacade;
import com.diagbot.vo.BillPageVO;
import com.diagbot.vo.BillSaveVO;
import com.diagbot.vo.ConceptIndexVO;
import com.diagbot.vo.ConditionIndexVO;
import com.diagbot.vo.RelationIdVO;
import com.diagbot.vo.RelationStatusVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/11/17 13:56
 */
@RestController
@RequestMapping("/billman")
@Api(value = "开单合理项维护相关API", tags = { "开单合理项维护相关API" })
@SuppressWarnings("unchecked")
public class BillManController {
    @Autowired
    BillManFacade billManFacade;

    @ApiOperation(value = "获取开单合理项列表[zhaops]",
            notes = "number:页码<br>" +
                    "size:每页条目数<br>" +
                    "types:开单类型（多选）1-检验套餐、2-检查、3-检查子项、4-手术和操作、5-药品注册名<br>" +
                    "conceptName:开单项（模糊匹配）<br>" +
                    "conditionName:条件明细（模糊匹配）<br>" +
                    "relationStatus:启用禁用标志（0-禁用，1-启用）<br>")
    @PostMapping("/getPage")
    public RespDTO<BillItem> getPage(@RequestBody BillPageVO billPageVO) {
        Page<BillItem> data = billManFacade.getPage(billPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "条件明细检索[zhaops]",
            notes = "type:开单类型（1-检验套餐、2-检查、3-检查子项、4-手术和操作、5-药品注册名）<br>" +
                    "conceptName:开单项<br>" +
                    "ruleType:规则类型（1-文本类型、2-数值类型）<br>" +
                    "conditionName:条件明细（模糊匹配）<br>")
    @PostMapping("/conditionIndex")
    public RespDTO<BillCondition> conditionIndex(@RequestBody ConditionIndexVO conditionIndexVO) {
        List<BillCondition> data = billManFacade.conditionIndex(conditionIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "开单项检索[zhaops]",
            notes = "type:开单类型（1-检验套餐、2-检查、3-检查子项、4-手术和操作、5-药品注册名）<br>" +
                    "conceptName:开单项（模糊匹配）<br>")
    @PostMapping("/conceptIndex")
    public RespDTO<BillConcept> conceptIndex(@RequestBody ConceptIndexVO conceptIndexVO) {
        List<BillConcept> data = billManFacade.conceptIndex(conceptIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "查询单条规则[zhaops]",
            notes = "relationId:关系id<br>")
    @PostMapping("/getRecordByRelationId")
    public RespDTO<BillSaveVO> getRecordByRelationId(@RequestBody @Valid RelationIdVO relationIdVO) {
        BillSaveVO data = billManFacade.getRecordByRelationId(relationIdVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存规则[zhaops]",
            notes = "")
    @PostMapping("/saveRecord")
    public RespDTO<Boolean> saveRecord(@RequestBody BillSaveVO billSaveVO) {
        Boolean data = billManFacade.saveRecord(billSaveVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "规则启用禁用[zhaops]",
            notes = "relationId:关系id<br>" +
                    "status:状态（0-禁用、1-启用)")
    @PostMapping("/updateStatus")
    public RespDTO<Boolean> updateStatus(@RequestBody @Valid RelationStatusVO relationStatusVO) {
        Boolean data = billManFacade.updateStatus(relationStatusVO);
        return RespDTO.onSuc(data);
    }
}